{% extends "base.html %}

{% block nav-help %}<li class="active"><a href="#">Help</a></li>{% end %}

{% block content %}
    <h2>Help</h2>
    <p>Arachnado is a tool to get all data from a website and store it in a database.</p>
{% end %}
