webpackJsonp([0],[function(e,t,n){"use strict";var s=n(1),o=(s.FancyWebSocket,n(3)),c=n(351);$(window).ready(function(){o.install("arachnado-connection-monitor"),c.installHeader("arachnado-process-stats")})},,,function(e,t,n){"use strict";function s(e){o.render(o.createElement(u,null),document.getElementById(e))}Object.defineProperty(t,"__esModule",{value:!0}),t.install=s;var o=n(4),c=n(160),i=n(182),r=i.Label,a=n(254),l=o.createClass({displayName:"ConnectionMonitorWidget",STATE_CLASSES:{offline:"danger",online:"info",crawling:"success"},render:function(){var e=this.STATE_CLASSES[this.props.status]||"default";return o.createElement(r,{bsStyle:e,title:"reconnect",style:{cursor:"pointer"},onClick:this.onClick},this.props.status)},onClick:function(){a.Actions.reconnect()}}),u=o.createClass({displayName:"ConnectionMonitor",mixins:[c.connect(a.store,"status")],render:function(){return o.createElement(l,{status:this.state.status})}})}]);